# ../sp.py

'''
    ===========================================================================
    Source Python
    Copyright (C) 2012 Source Python Development Team.  All rights reserved.
    ===========================================================================

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License, version 3.0, as
    published by the Free Software Foundation.
    
    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along
    with this program.  If not, see <http://www.gnu.org/licenses/>.

    As a special exception, the Source.Python Team gives you permission 
    to link the code of this program (as well as its derivative works) to 
    "Half-Life 2," the "Source Engine," and any Game MODs that run on software
    by the Valve Corporation.  You must obey the GNU General Public License in
    all respects for all other code used.  Additionally, the Source.Python
    Development Team grants this exception to all derivative works.  
'''

# =============================================================================
# >> IMPORTS
# =============================================================================
# Python Imports
#   Logging
from logging import getLogger

# Source.Python Imports
#   Addons
from addons.info import AddonInfo
from addons.manager import AddonManager
#   Events
from events.manager import EventRegistry
#   Logging (loaded in order to setup logging)
import _logging

# =============================================================================
# >> GLOBALS
# =============================================================================
# Set up logging for this module
_log = getLogger('sp')

# =============================================================================
# >> CORE FUNCTIONS
# =============================================================================
def addon_load(addon_name):
    '''Called when a user executes sp_load.'''

    # Is an addon being loaded?
    if not addon_name:

        # Print start message for loaded addons
        print('[SP] Loaded Addons:')
        print('======================================\n')

        # Loop through all loaded addons
        for addon in AddonManager:

            # Set info as None in case no AddonInfo instance is found
            info = None

            # Loop through the addon's globals
            for object_name in AddonManager[addon].globals:

                # Get the object's instance
                instance = AddonManager[addon].globals[object_name]

                # Is the current instance an AddonInfo instance?
                if isinstance(instance, AddonInfo):

                    # Set info to the instance
                    info = instance

                    # Break the loop
                    break

            # Does the addon have an AddonInfo instance?
            if not info is None:

                # Print the addon's name
                print(addon + ':')

                # Loop through all items in the AddonInfo instance
                for item in info:

                    # Print the item's name
                    print('\t%s:' % item)

                    # Print the item's value
                    print('\t\t%s' % info[item])

            # Does the addon not have an AddonInfo instance?
            else:

                # Print the addon's name
                print(addon)

            # Print a blank line between addons
            print('\n')

        # Print closing message for loaded addons
        print('======================================')

        # No need to go further
        return

    # Is the addon already loaded?
    if addon_name in AddonManager:

        # Log message that the addon is already loaded
        _log.warning('Addon "%s" is already loaded.', addon_name)

        # No need to go further
        return

    # Get the addon's instance
    addon = AddonManager[addon_name]

    # Is the addon loaded?
    if addon is None:

        # Log message that the addon was not able to be loaded
        _log.error('Addon "%s" was unable to be loaded.', addon_name)

        # If not, go no further
        return

    # Log message that the addon successfully loaded
    _log.info('Successfully loaded "%s"', addon_name)


def addon_unload(addon_name):
    '''Called when a user executes sp_unload.'''

    # Is the loaded?
    if not addon_name in AddonManager:

        # Log message that the addon is not loaded
        _log.warning('''Addon "%s" cannot be unloaded. It is not 
currently loaded.''', addon_name)

        # No need to go further
        return

    # Get the addon's instance
    addon = AddonManager[addon_name]

    # Remove the addon from the AddonManager
    del AddonManager[addon_name]

    # Log message that the addon successfully unloaded
    _log.info('Successfully unloaded "%s"', addon_name)


def addon_reload(addon_name):
    '''Called when a user executes sp_reload.'''

    # Unload the addon
    addon_unload(addon_name)

    # Load the addon
    addon_load(addon_name)


def event_fire(GameEvent):
    '''Called when the core catches an event.'''

    # Call the event within the registry
    EventRegistry.CallEventCallbacks(GameEvent)
